//Testroutine fr Steuerung: Nach Einschalten werden die Eingnge 3x ein und ausgeschalten, 
//danach kann mit den Eingabetasten der jeweilige Ausgang getestet werden, auerdem luft ein Testprogramm
//fr das LCD an Port C

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>
#include "lcd.h"
#include "clock.h"
#include "usart.h"


#define FALSE 0
#define TRUE 1

#define BLINKDAUER 8							//Dauer der Blinkroutine
#define TI_BLINK 1								//Blinkfrequenz 1Hz
#define TI_INPUT 1								//Tasktimer der INPUT Routine
#define TI_DCF 1			                	//Tasktimer der LCD Testroutine


#define KEY1  (1<<PA0)
#define KEY2  (1<<PA1)
#define KEY3  (1<<PA2)
#define KEY4  (1<<PA3)
#define KEY5  (1<<PA4)
#define KEY6  (1<<PA5)
#define KEY7  (1<<PA6)
#define KEY8  (1<<PA7)
#define KEY9  (1<<PC7)

#define OUTPUT1  (1<<PB0)
#define OUTPUT2	 (1<<PB1) 	
#define OUTPUT3  (1<<PB2)
#define OUTPUT4	 (1<<PB3)	
#define OUTPUT5  (1<<PB4)
#define OUTPUT6  (1<<PB5)
#define OUTPUT7  (1<<PB6)
#define OUTPUT8  (1<<PB7)



volatile uint8_t blinkTimer;					//Timer fr Blinkfrequenz
volatile uint8_t wartezeit = BLINKDAUER;		//Blinkdauer zu Beginn auf 3 sec laden
volatile uint8_t INPUT_TIMER;					//Timer fr Eingabetestfunktion
volatile uint8_t DCF_TIMER;


void BLINK_TEST (void);							//Deklaration der Blinkfunktion	
void INPUT_TEST (void);							//Deklaration der Eingabetestfunktion
void LDCF_ANZEIGE (void);							//Deklaration der LCD Testfunktion







ISR(TIMER0_OVF_vect) 					// Interrupt alle Millisekunde
{
	static uint16_t sekunden = 1000;

	TCNT0=131;							//Preloader um 1ms zu erreichen

	
	
	
	//Millisekunden Timer---------------

	//Tasktimer dekrementieren bis sie null sind
	if (INPUT_TIMER) INPUT_TIMER--;
	if (DCF_TIMER) DCF_TIMER--;	


	

	
	if (--sekunden == 0)				//zuerst dekrementieren, dann schauen ob 0
	{
		sekunden = 1000;				//1000 ms laden um Sekundentakt zu erreichen
		if (wartezeit) wartezeit--;
		if (blinkTimer) blinkTimer --;
		
			//Wartezeit dekrementieren wenn nicht null
		
	
	}
		

}

int main(void)
{
	//---- Initialisierung Port IO -------------------------------

	DDRB = 0xFF;						// Ganzen Port B auf Ausgang schalten
	PORTB = 0xFF;						// Initialisieren der Richtungsanzeige

	DDRC = 0x80;     					// Pin 8 von Port C auf Eingang, Rest auf Ausgang fr LCD

	
	// --- Timer 0 Initialisierung (fr ATMega8515 im Normal Mode) ---------------------------------------
	//
	
	TCCR0= (1<<CS01) | (1<<CS00); 		// Mode: Normal, Timer 0 Takt: CLK / 64			
	TCNT0 = 131;				  		// Count Register laden (17,36 s * 58 ergibt ca. 1 ms)
	TIMSK = (1<<TOIE0);			  		// Timer 0 Output Compare Interrupt freigeben		

	
	Start_Clock();

	lcd_init(LCD_DISP_ON);		  		//Initialisierung LCD		
	lcd_clrscr();		  	

       	
	
	sei();						  		// globale Interruptfreigabe



	while (1)	{

		if (wartezeit)				//wenn Timer fr Bklinkroutine gleich null
			BLINK_TEST();				//Blinktaskt ausfhren
		else
		{
			if (INPUT_TIMER ==0)			//wenn Tasktimer fr Eingabetest gleich null
			{	
				INPUT_TIMER = TI_INPUT;		//Tasktimer fr Eingabetest auf 20ms	
				INPUT_TEST();
			}
		}
		if (DCF_TIMER ==0)				//wenn Tasktimer fr LCD Test gleich null
		{
			DCF_TIMER = TI_DCF;			//Tasktimer wieder laden	
			DCF_ANZEIGE();					//LCD Test ausfhren
		}
	
	}

	return 0;
}

//---------------------------Blinktask------------------------------------------------------------------

		void BLINK_TEST (void)
		{
			if (blinkTimer == 0)
			
				{
					blinkTimer = TI_BLINK;  
					PORTB = ~PORTB;	 
				}
		
//				else if (wartezeit == 0)

//				{
//					PORTB = 0x00;
//
//				}

		}

//---------------------------Eingabetest-----------------------------------------------------------------

		void INPUT_TEST (void)

		{
		
			uint8_t BUTTONS;
			uint8_t BUTTON_9;

			BUTTONS = ~PINA;
			BUTTON_9= PINC;

			switch(BUTTONS)

			{

				case KEY1:

					PORTB |= OUTPUT1;
					break;
		
				case KEY2:

					PORTB |= OUTPUT2;
					break;

				case KEY3: 
			
					PORTB |= OUTPUT3;
					break;

				case KEY4:

					PORTB |= OUTPUT4;
					break;

				case KEY5:

					PORTB |= OUTPUT5;
					break;

				case KEY6:

					PORTB |= OUTPUT6;
					break;

				case KEY7:

					PORTB |= OUTPUT7;
					break;

				case KEY8:

					PORTB |= OUTPUT8;
					break;

				default:

					PORTB = 0x00;
					break;
			}
//-----------------------------------LCD Testroutine----------------------------------------------------------			
		
		}

		void DCF_ANZEIGE (void)

		{
				
			char buffer[10];
			char temp_buffer[10];

			lcd_gotoxy(0,0);
			lcd_puts("Time:");
			itoa(hh,buffer,10);
			lcd_gotoxy(6,0);
			if (hh<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(8,0);
			lcd_puts(":");

			itoa(mm,buffer,10);
			if (mm<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(11,0);
			lcd_puts(":");

			itoa(ss,buffer,10);
			if (ss<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(12,0);
		
				
			itoa (flags.dcf_sync, buffer, 10);
			if (flags.dcf_sync)
				{
					lcd_gotoxy(4,3);
					lcd_puts("OK    ");
				}
			else
				{
					lcd_gotoxy(4,3);
					lcd_puts("NO SIG");
				}

			
			lcd_gotoxy(0,1);
			lcd_puts("Date:");
			itoa (day, buffer, 10);
			lcd_gotoxy(6,1);
			if (day<10)
			lcd_puts ("0");
			lcd_puts(buffer);
			lcd_gotoxy(8,1);
			lcd_puts(".");

			itoa (mon, buffer, 10);
			if (mon<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(11,1);
			lcd_puts(".");

			itoa(year, buffer, 10);
			if (year<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(15,3);
			
		
			itoa (flags.dcf_rx, buffer, 10);
			lcd_puts (buffer);


			
			lcd_gotoxy(0,2);
			lcd_puts("Water Temp:");
			lcd_gotoxy(15,2);
			lcd_puts("C");
			lcd_gotoxy(0,3);
			lcd_puts("DCF:");

			
				

			
		}

		


			
